#-----
#Import Robot Points
#
#Import data in csv as motion statements into robot program.
#-----
#Version 1.0
#-----
#CSV file can have the following fields (in any order dictated by the first line) where only X,Y,Z are mandatory:
#
#Name,X,Y,Z,RX,RY,RZ,Base,Tool,Motion,MaxSpeed,JointSpeed,AccuracyValue,E1,E2,E3,E4,E5,E6
#P1,100.0,200.0,300.0,0,90,0,Base1,Gripper,LIN,1000.0,100.0,5.0,111.1,222.2,0,0,0,0
#-----

from vcCommand import *
import vcMatrix, vcVector

app = getApplication()
cmd = getCommand()
#-----
def OnStart():
  rou = app.TeachContext.ActiveRoutine
  if not rou:
    print 'No robot selected.'
    return
  
  open_dialog = app.findCommand('dialogOpen')
  open_dialog.OnPostExecute = readCSV
  open_dialog.Param_3 = 'CSV Files (*.csv)|*.csv'
  open_dialog.execute()
#-----
def readCSV(open_dialog):
  open_dialog.OnPostExecute = None
  uri = open_dialog.Param_1[8:]
  
  rou = app.TeachContext.ActiveRoutine
  exe = rou.Program.Executor
  ctr = exe.Controller
  mt_init = ctr.createTarget()
  separator = ','
  
  with open(uri, 'r') as csv_file:
  
    #Try to find separator character from first line
    line = csv_file.readline()
    line = line.rstrip('\n')
    line = line.lower()
    count = 0
    trythese = ',;:'
    for c in trythese:
      if line.count(c) > count:
        count = line.count(c)
        separator = c
      #endif
    #endfor
    
    #Find out data headers on the first line
    fields = line.split(separator)
    if not 'x' in fields or not 'y' in fields or not 'z' in fields:
      #No coordinate data (only necessary fields)
      print 'No XYZ data found in the header. Aborting.'
      return
      
    #Iterate data lines
    index = 0
    for line in csv_file:
      index +=1
      
      line = line.rstrip('\n')
      data = line.split(separator)
      
      name = 'P' + str(index)
      x = 0
      y = 0
      z = 0
      w = 0
      p = 0
      r = 0
      base = getBase(ctr,mt_init.BaseName)
      tool = getTool(ctr,mt_init.ToolName)
      motion = VC_STATEMENT_LINMOTION
      max_speed = 1000.0
      joint_speed = 1.0
      accuracy = 0.0
      ex_joints = []
      
      try:
        x = float(data[fields.index('x')])
        y = float(data[fields.index('y')])
        z = float(data[fields.index('z')])
        
        if 'name' in fields:
          name = data[fields.index('name')]
        
        if 'rx' in fields and 'ry' in fields and 'rz' in fields:
          w = float(data[fields.index('rx')])
          p = float(data[fields.index('ry')])
          r = float(data[fields.index('rz')])
        elif 'w' in fields and 'p' in fields and 'r' in fields:
          w = float(data[fields.index('w')])
          p = float(data[fields.index('p')])
          r = float(data[fields.index('r')])
        elif 'a' in fields and 'b' in fields and 'c' in fields:
          #A=RZ, B=RY, C=RX
          w = float(data[fields.index('c')])
          p = float(data[fields.index('b')])
          r = float(data[fields.index('a')])
          
        if 'base' in fields:
          base = getBase(ctr,data[fields.index('base')])
        if 'tool' in fields:
          tool = getTool(ctr,data[fields.index('tool')])
        
        if 'motion' in fields:
          if data[fields.index('motion')].lower() == 'ptp':
            motion = VC_STATEMENT_PTPMOTION
            
        if 'maxspeed' in fields:
          max_speed = float(data[fields.index('maxspeed')])
          
        if 'jointspeed' in fields:
          joint_speed = float(data[fields.index('jointspeed')]) / 100.0
          
        if 'accuracyvalue' in fields:
          accuracy = float(data[fields.index('accuracyvalue')])
          
        for i in range(6):
          e_name = 'e' + str(i+1)
          if e_name in fields:
            ex_joints.append(float(data[fields.index(e_name)]))
        
      except:
        print 'Invalid data line, skipping.'
        continue
      
      m = vcMatrix.new()
      m.translateAbs(x,y,z)
      m.setWPR(w,p,r)
      
      stat = rou.addStatement(motion)
      stat.readIn()
      stat.Positions[0].Name = name
      stat.Positions[0].PositionInReference = m
      stat.Base = base
      stat.Tool = tool
      stat.AccuracyMethod = VC_MOTIONTARGET_AM_DISTANCE
      stat.AccuracyValue = accuracy
      
      if motion == VC_STATEMENT_LINMOTION:
        stat.MaxSpeed = max_speed
      else:
        stat.JointSpeed = joint_speed
        
        
        
      for i in range(len(ex_joints)):
        e_name = 'E' + str(i+1)
        prop = stat.Positions[0].getProperty(e_name)
        if prop:
          prop.Value = ex_joints[i]
    #endfor line in csv_file:
  #endwith open(uri, 'r') as csv_file:
    
  app.render()
  print 'Done.'
#-----
def getBase(ctr, base_name):
  for base in ctr.Bases:
    if base.Name == base_name:
      return base
  return None
#-----
def getTool(ctr, tool_name):
  for tool in ctr.Tools:
    if tool.Name == tool_name:
      return tool
  return None
#-----